local ADDON_NAME, MRH = ...;

-- WoW API
local GetSpellInfo = GetSpellInfo;
local UnitGUID, UnitExists, UnitClass, GetUnitName = UnitGUID, UnitExists, UnitClass, GetUnitName;
local SendChatMessage, C_ChatInfo_SendAddonMessage, C_ChatInfo_RegisterAddonMessagePrefix = SendChatMessage, C_ChatInfo.SendAddonMessage, C_ChatInfo.RegisterAddonMessagePrefix;

local encounterData = {
    currentID     = nil,
    mekkatorqueID = 2276,
};

-- Used to know what players are in robots (vehicles)
local targetUnit, vehicleGUIDs = nil, {};

-- Robo-buttons data
local buttonsData = { 
    [1] = { whisper = '{rt7} {rt7} [1] RED {rt7} {rt7}',    spellID = 286152 },
    [2] = { whisper = '{rt3} {rt3} [2] PURPLE {rt3} {rt3}', spellID = 286192 },
    [3] = { whisper = '{rt4} {rt4} [3] GREEN {rt4} {rt4}',  spellID = 286215 },
    [4] = { whisper = '{rt1} {rt1} [4] YELLOW {rt1} {rt1}', spellID = 286219 },
    [5] = { whisper = '{rt6} {rt6} [5] BLUE {rt6} {rt6}',   spellID = 286226 },
};

-- Prefixes
local ADDON_PREFIXES = {
    ['MEKKAHELPER']   = true,
    ['ST_MEKKAHLPER'] = true,
};

-- Register prefixes
do
    for prefix, state in pairs(ADDON_PREFIXES) do
        if state then
            C_ChatInfo_RegisterAddonMessagePrefix(prefix);
        end
    end
end

local MRHFrame = CreateFrame('Frame', 'MekkaRoboHelper', UIParent);
MRHFrame:SetPoint('CENTER', ExtraActionBarFrame, 'CENTER', 0, 70);
MRHFrame:SetSize(198, 70);
MRHFrame:SetFrameStrata('HIGH');
MRHFrame:SetMovable(true);

MRHFrame.dragger = CreateFrame('Frame', nil, MRHFrame);
MRHFrame.dragger:SetPoint('TOPLEFT', MRHFrame, 'TOPLEFT', 0, 0);
MRHFrame.dragger:SetPoint('TOPRIGHT', MRHFrame, 'TOPRIGHT', 0, 0);
MRHFrame.dragger:SetSize(MRHFrame:GetWidth(), 28);
MRHFrame.dragger:EnableMouse(true);
MRHFrame.dragger:RegisterForDrag('LeftButton');
MRHFrame.dragger:SetScript('OnDragStart', function(self, button) MRHFrame:StartMoving(); end);
MRHFrame.dragger:SetScript('OnDragStop', function(self) MRHFrame:StopMovingOrSizing(); end);

MRHFrame.background = MRHFrame:CreateTexture(nil, 'BACKGROUND', nil, -7);
MRHFrame.background:SetTexture('Interface\\Buttons\\WHITE8x8');
MRHFrame.background:SetAllPoints(MRHFrame);
MRHFrame.background:SetVertexColor(0.1, 0.1, 0.1, 1);

MRHFrame.border = MRHFrame:CreateTexture(nil, 'BACKGROUND', nil, -8);
MRHFrame.border:SetTexture('Interface\\Buttons\\WHITE8x8');
MRHFrame.border:SetPoint('TOPLEFT', MRHFrame, 'TOPLEFT', -2, 2);
MRHFrame.border:SetPoint('BOTTOMRIGHT', MRHFrame, 'BOTTOMRIGHT', 2, -2);
MRHFrame.border:SetVertexColor(0, 0, 0, 1);

MRHFrame:Hide();

MRHFrame.targetName = MRHFrame:CreateFontString(nil, 'BACKGROUND', 'SystemFont_Outline');
MRHFrame.targetName:SetPoint('TOP', MRHFrame, 'TOP', 0, -4);
MRHFrame.targetName:SetText('|cffccccccNo Target|r');

MRHFrame.buttons = {};
   
for i, v in ipairs(buttonsData) do
    MRHFrame.buttons[i] = MRHFrame.buttons[i] or CreateFrame('Button', nil, MRHFrame);
    MRHFrame.buttons[i]:SetPoint('LEFT', MRHFrame, 'LEFT', (i-1)*36 + (i-1)*3 + 5, -14);
    MRHFrame.buttons[i]:SetSize(32, 32);

    MRHFrame.buttons[i].texture = MRHFrame.buttons[i]:CreateTexture(nil, 'BACKGROUND');
    MRHFrame.buttons[i].texture:SetAllPoints();
    MRHFrame.buttons[i].texture:SetTexture(select(3, GetSpellInfo(v.spellID)));
    MRHFrame.buttons[i].texture:SetTexCoord(0.1, 0.9, 0.1, 0.9);

    MRHFrame.buttons[i].border = MRHFrame.buttons[i]:CreateTexture(nil, 'BACKGROUND', nil, -8);
    MRHFrame.buttons[i].border:SetTexture('Interface\\Buttons\\WHITE8x8');
    MRHFrame.buttons[i].border:SetPoint('TOPLEFT', MRHFrame.buttons[i], 'TOPLEFT', -2, 2);
    MRHFrame.buttons[i].border:SetPoint('BOTTOMRIGHT', MRHFrame.buttons[i], 'BOTTOMRIGHT', 2, -2);
    MRHFrame.buttons[i].border:SetVertexColor(0.3, 0.3, 0.3, 1);

    MRHFrame.buttons[i].whisper = v.whisper;
    MRHFrame.buttons[i].button_id = i;

    MRHFrame.buttons[i]:SetScript('OnClick', function(self, button)
        if not targetUnit then return; end
        
        if UnitExists(targetUnit) then
            local fullName = GetUnitName(targetUnit, true);

            SendChatMessage(self.whisper, 'WHISPER', nil, fullName);

            for prefix, state in pairs(ADDON_PREFIXES) do
                if state then
                    C_ChatInfo_SendAddonMessage(prefix, self.button_id, 'WHISPER', fullName);
                end
            end
        end  
    end);
end

MRHFrame.receiver = CreateFrame('Frame', 'MekkaRoboHelperReceiver', UIParent); 
MRHFrame.receiver:SetPoint('CENTER', 0, 150);
MRHFrame.receiver:SetClampedToScreen(true);
MRHFrame.receiver:SetSize(60, 60);

MRHFrame.receiver.border = MRHFrame.receiver:CreateTexture(nil, 'BACKGROUND', nil, -8);
MRHFrame.receiver.border:SetTexture('Interface\\Buttons\\WHITE8x8');
MRHFrame.receiver.border:SetPoint('TOPLEFT', MRHFrame.receiver, 'TOPLEFT', -2, 2);
MRHFrame.receiver.border:SetPoint('BOTTOMRIGHT', MRHFrame.receiver, 'BOTTOMRIGHT', 2, -2);
MRHFrame.receiver.border:SetVertexColor(1, 1, 0.75, 1);

MRHFrame.receiver.texture = MRHFrame.receiver:CreateTexture(nil, 'ARTWORK');
MRHFrame.receiver.texture:SetAllPoints();
MRHFrame.receiver.texture:SetSize(60, 60);
MRHFrame.receiver.texture:SetTexture('Interface\\ICONS\\INV_Misc_QuestionMark');
MRHFrame.receiver.texture:SetTexCoord(0.1, 0.9, 0.1, 0.9);

MRHFrame.receiver.number = MRHFrame.receiver:CreateFontString(nil, 'OVERLAY', 'SystemFont_OutlineThick_Huge4');
MRHFrame.receiver.number:SetPoint('TOPRIGHT', MRHFrame.receiver, 'TOPRIGHT', -2, -6);

MRHFrame.receiver:Hide();

local function getUnitInVehicle(GUID)
    if not GUID then
        return nil;
    end
    
    return vehicleGUIDs[GUID];
end

local function getVehicleOfUnit(unit)
    if not unit then
        return nil;
    end
    
    for k, v in pairs(vehicleGUIDs) do
        if v == unit then
            return k;
        end
    end
    
    return nil;
end

local events = CreateFrame('Frame');
events:RegisterEvent('ENCOUNTER_START');
events:SetScript('OnEvent', function(self, event, ...) return self[event](self, event, ...); end);

function events:ENCOUNTER_START(_, eID)
    encounterData.currentID = tonumber(eID);
        
    if encounterData.currentID and encounterData.currentID == encounterData.mekkatorqueID then
        self:RegisterEvent('ENCOUNTER_END');
        self:RegisterEvent('PLAYER_TARGET_CHANGED');
        self:RegisterEvent('UNIT_ENTERED_VEHICLE');
        self:RegisterEvent('UNIT_EXITED_VEHICLE');
        self:RegisterEvent('CHAT_MSG_ADDON');
        self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED');
    end
end

function events:ENCOUNTER_END()
    if encounterData.currentID and encounterData.currentID == encounterData.mekkatorqueID then
        self:UnregisterEvent('ENCOUNTER_END');
        self:UnregisterEvent('PLAYER_TARGET_CHANGED');
        self:UnregisterEvent('UNIT_ENTERED_VEHICLE');
        self:UnregisterEvent('UNIT_EXITED_VEHICLE');
        self:UnregisterEvent('CHAT_MSG_ADDON');
        self:UnregisterEvent('UNIT_SPELLCAST_SUCCEEDED');
    end

    encounterData.currentID = nil;
    vehicleGUIDs = {};

    MRHFrame:Hide();
    MRHFrame.receiver:Hide();
end

function events:PLAYER_TARGET_CHANGED()
    local targetGUID = UnitGUID('target');

    targetUnit = getUnitInVehicle(targetGUID);

    if targetUnit then
        MRHFrame:Show();

        local class_color = RAID_CLASS_COLORS[select(2, UnitClass(targetUnit)) or 'PRIEST'].colorStr;
        MRHFrame.targetName:SetText('|c' .. class_color .. GetUnitName(targetUnit) .. '|r');
    else
        MRHFrame:Hide();
    end
end

function events:UNIT_ENTERED_VEHICLE(_, unit, ...)
    local vehicleGUID = select(4, ...);

    if vehicleGUID then
        vehicleGUIDs[vehicleGUID] = unit;
    end
end

function events:UNIT_EXITED_VEHICLE(_, unit)
    local vehicleGUID = getVehicleOfUnit(unit);

    if vehicleGUID then 
        vehicleGUIDs[vehicleGUID] = nil;
    end

    if unit == 'player' then
        MRHFrame.receiver:Hide();
    end
end

function events:CHAT_MSG_ADDON(_, eventPrefix, buttonID)
    for prefix, state in pairs(ADDON_PREFIXES) do
        if state and eventPrefix == prefix then
            local buttonID = tonumber(buttonID);

            PlaySound(SOUNDKIT.PUT_DOWN_GEMS, 'Master');

            MRHFrame.receiver.texture:SetTexture(select(3, GetSpellInfo(buttonsData[buttonID].spellID)));
            MRHFrame.receiver.number:SetText(buttonID);

            MRHFrame.receiver:Show();

            break;
        end
    end
end

function events:UNIT_SPELLCAST_SUCCEEDED(_, unit, _, spellID)
    local spellID = tonumber(spellID);

    if unit == 'player' then
        for i = 1, 5 do
            if spellID == buttonsData[i].spellID then
                MRHFrame.receiver:Hide();
                break;
            end
        end
    end
end